<?php

class Biteship_Shipping_Method extends WC_Shipping_Method {
    public function __construct() {
        $this->id                 = 'biteship';
        $this->title       = __( 'Biteship' );
        $this->method_title = __('Biteship');
        $this->method_description = __( 'Description of your shipping method' ); // 
        $this->shipping_calculation_error = '';
        $this->option_list = array( 'shipping_service_enabled', 'store_address', 'store_addresses', 'default_weight' );
        $this->init();
    }

    function init() {
        // Load the settings API
        $this->init_form_fields(); // This is part of the settings API. Override the method to add your own settings
        $this->init_settings(); // This is part of the settings API. Loads settings you previously init.

        // Save settings in admin if you have any defined
        add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );

        $this->rest_adapter = new Biteship_Rest_Adapter($this->settings['license']);
    }

    function init_form_fields() {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __( 'Enable', 'biteship' ),
                'type' => 'checkbox',
                'description' => __( 'Activate Biteship plugin', 'biteship' ),
                'default' => 'yes',
            ),
            'license' => array(
                'title' => __( 'License', 'biteship' ),
                'type' => 'textarea',
                'description' => __( 'Biteship license', 'biteship' ),
            ),
            'gmap_api_key' => array(
                'title' => __( 'Google Maps Api Key', 'biteship' ),
                'type' => 'text',
                'description' => __( 'This is needed to load map for Gosend and Grab service', 'biteship' ),
            ),
        );
    }

    function reset_settings_and_option() {
        foreach($this->option_list as $key) {
            delete_option($this->get_biteship_option_key($key));
        }

        delete_option('woocommerce_' . $this->id . '_settings');
    }

    public function calculate_shipping( $package = array() ) {
        $this->shipping_calculation_error = '';
        if ($this->settings['enabled'] == 'yes') {
            $options = $this->get_options();
            $shipping_service_enabled = $options['shipping_service_enabled'];

            if (is_array($shipping_service_enabled)) {
                $couriers = array();
                foreach($shipping_service_enabled as $service) {
                    $courier = explode("/", $service)[0];
                    if (!in_array($courier, $couriers)) {
                        array_push($couriers, $courier);
                    }
                }

                $items = array();
                $default_weight = $options['default_weight'];
                foreach($package['contents'] as $item) {
                    $item_data = $item['data'];
                    $weight = $item_data->has_weight() ? $item_data->get_weight() : $default_weight;
                    array_push($items, array(
                        'name' => $item_data->get_name(),
                        'length' => $this->get_dimension_in_cm($item_data->get_length()),
                        'width' => $this->get_dimension_in_cm($item_data->get_width()),
                        'height' => $this->get_dimension_in_cm($item_data->get_height()),
                        'weight' => $this->get_weight_in_gram($weight),
                        'quantity' => $item['quantity'],
                        'value' => $item_data->get_regular_price(),
                    ));
                }

                $address_id = $options['store_address'];
                $origin_zip_code = $this->get_store_zipcode($options['store_addresses'], $address_id);
                $destination_zip_code = $this->get_destination_zipcode();
                $store_position = $this->get_store_position($options['store_addresses'], $address_id);

                $query = array(
                    'origin_latitude' => $this->get_store_latitude($store_position),
                    'origin_longitude' => $this->get_store_longitude($store_position),
                    'destination_latitude' => $package['destination']['latitude'],
                    'destination_longitude' => $package['destination']['longitude'],
                    'requested_services' => $shipping_service_enabled,
                    'origin_postal_code' => $origin_zip_code,
                    'destination_postal_code' => $destination_zip_code,
                    'couriers' => $couriers,
                    'items' => $items,
                );

                $rates = $this->rest_adapter->get_pricing($query);
                if (!is_array($rates)) {
                    $this->shipping_calculation_error = $rates;
                    return;
                }

                foreach($rates as $rate) {
                    $this->add_rate($rate);
                }
            }
        }
    }

    public function admin_options() {
        $this->save_settings();

        $companies = $this->rest_adapter->get_couriers();
        $options = $this->get_options();
        require_once BITESHIP_PLUGIN_PATH . 'admin/views/settings.php';
    }

    public function get_options() {
        $options = array();
        foreach($this->option_list as $key) {
            $options[$key] = get_option($this->get_biteship_option_key($key));
        }

        return $options;
    }

    private function save_settings() {
        if (wp_verify_nonce( @$_REQUEST['biteship-nonce'], 'biteship-settings' )) {
            $_POST['shipping_service_enabled'] = array();
            if ( isset($_POST['shipping_company_checkbox']) ) {
                foreach( $_POST['shipping_company_checkbox'] as $id => $val ) {
                    array_push($_POST['shipping_service_enabled'], $id);
                }
            }

            $_POST['store_addresses'] = get_option($this->get_biteship_option_key('store_addresses'));
            if ( isset($_POST['new_address']) && isset($_POST['new_zipcode']) ) {
                $address = sanitize_text_field( $_POST['new_address'] );
                $zipcode = sanitize_text_field( $_POST['new_zipcode'] );
                if ($address != '' && $zipcode != '') {
                    $position = '';
                    if ( isset($_POST['new_position']) ) {
                        $position = sanitize_text_field( $_POST['new_position'] );
                    }

                    $new_store_address = array(
                        'id' => wp_generate_uuid4(),
                        'address' => $address,
                        'zipcode' => $zipcode,
                        'position' => $position,
                    );
                    if (!is_array($_POST['store_addresses'])) {
                        $_POST['store_addresses'] = array();
                    }
                    array_push($_POST['store_addresses'], $new_store_address);
                }
            }

            if ( isset($_POST['remove_store_address']) ) {
                $removed_id = sanitize_text_field( $_POST['remove_store_address'] );
                if ($removed_id != '') {
                    foreach($_POST['store_addresses'] as $key => $sd) {
                        if ($sd['id'] == $removed_id) {
                            unset($_POST['store_addresses'][$key]);
                        }
                    }

                    if ( $removed_id == $_POST['store_address'] ) {
                        if (count($_POST['store_addresses']) > 0) {
                            $_POST['store_address'] = array_values($_POST['store_addresses'])[0]['id'];
                        }
                    }
                }
            }

            $new_options = array();
            foreach( $this->option_list as $opt_key ) {
                $new_options[$opt_key] = $_POST[$opt_key];
            }
            $this->save_options($new_options);
        }
    }

    private function save_options($new_options) {
        $old_options = $this->get_options();
        foreach( $old_options as $key => $old_value ) {
            if ( !isset($new_options[$key]) ) {
                continue;
            }

            $new_value = $new_options[$key];
            if ( is_array($new_value)) {
                foreach( $new_value as $val ) {
                    sanitize_text_field($val);
                }
            } else {
                sanitize_text_field($new_value);
            }

            if ($new_value != $old_value) {
                update_option($this->get_biteship_option_key($key), $new_value);
            }
        }
    }

    private function get_biteship_option_key($key) {
        return $this->id . '_' . $key;
    }

    private function is_service_checked($service_code, $shipping_service_enabled) {
        if (is_array($shipping_service_enabled)) {
            return in_array($service_code, $shipping_service_enabled);
        }

        return false;
    }

    private function get_weight_in_gram($weight) {
        $weight_unit = get_option('woocommerce_weight_unit');

        switch ($weight_unit) {
            case 'kg':
                return $weight * 1000;
            case 'lbs':
                return $weight * 453.592;
            case 'oz':
                return $weight * 28.3495;
            default:
                return $weight;
        }
    }

    private function get_dimension_in_cm($dimension) {
        $dimension_unit = get_option('woocommerce_dimension_unit');
        switch ($dimension_unit) {
            case 'm':
                return $dimension * 100;
            case 'mm':
                return $dimension / 10;
            case 'in':
                return $dimension * 2.54;
            case 'yd':
                return $dimension * 91.44;
            default:
                return $dimension;
        }
    }

    private function get_store_zipcode($store_addresses, $active_id) {
        foreach($store_addresses as $address) {
            if ($address['id'] == $active_id) {
                return $address['zipcode'];
            }
        }

        return '';
    }

    private function get_store_position($store_addresses, $id) {
        foreach($store_addresses as $address) {
            if ($address['id'] == $id) {
                return $address['position'];
            }
        }

        return '';
    }

    private function get_store_latitude($position) {
        $tmp = explode(',', $position);
        if (count($tmp) > 1) {
            return $tmp[0];
        }

        return '';
    }

    private function get_store_longitude($position) {
        $tmp = explode(',', $position);
        if (count($tmp) > 1) {
            return $tmp[1];
        }

        return '';
    }

    private function get_destination_zipcode() {
        $field_names = array('calc_shipping_postcode', 'billing_postcode', 'shipping_postcode');

        foreach ($field_names as $field) {
            $zipcode = WC()->checkout->get_value($field);
            if ($zipcode != '') {
                return $zipcode;
            }
        }

        return '';
    }
}