<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://habibridho.github.io
 * @since      1.0.0
 *
 * @package    Biteship
 * @subpackage Biteship/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Biteship
 * @subpackage Biteship/includes
 * @author     Habib <habibridho@live.com>
 */
class Biteship {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Biteship_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'BITESHIP_VERSION' ) ) {
			$this->version = BITESHIP_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'biteship';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Biteship_Loader. Orchestrates the hooks of the plugin.
	 * - Biteship_i18n. Defines internationalization functionality.
	 * - Biteship_Admin. Defines all hooks for the admin area.
	 * - Biteship_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-biteship-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-biteship-i18n.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-biteship-rest-adapter.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-biteship-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-biteship-public.php';

		$this->loader = new Biteship_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Biteship_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Biteship_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Biteship_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'plugins_loaded', $plugin_admin, 'on_loaded');

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Biteship_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

		$this->loader->add_action( 'woocommerce_shipping_init', $this, 'load_shipping_method');
		$this->loader->add_filter( 'woocommerce_shipping_methods', $this, 'register_shipping_methods');

		$this->loader->add_action( 'woocommerce_checkout_before_order_review', $this, 'checkout_map_field' );
		$this->loader->add_action( 'woocommerce_checkout_fields', $this, 'custom_position_field' );

		$this->loader->add_filter( 'woocommerce_cart_no_shipping_available_html', $this, 'override_no_shipping_text' );
		$this->loader->add_filter( 'woocommerce_no_shipping_available_html', $this, 'override_no_shipping_text' );
		$this->loader->add_filter( 'woocommerce_cart_shipping_packages', $this, 'handle_shipping_position' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Biteship_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	public function load_shipping_method() {
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-biteship-shipping-method.php';
	}

	public function register_shipping_methods( $methods ) {
		$methods['biteship'] = 'Biteship_Shipping_Method';
		return $methods;
	}

	public function override_no_shipping_text($default) {
		$shipping_methods = WC()->shipping()->get_shipping_methods();
		$biteship_shipping = $shipping_methods['biteship'];
		if ($biteship_shipping != null) {
			$error = $biteship_shipping->shipping_calculation_error;
			if ($error != '') {
				return $error;
			}
		}

		return $default;
	}

	public function checkout_map_field( $checkout ) {
		$shipping_methods = WC()->shipping()->get_shipping_methods();
		$biteship_shipping = $shipping_methods['biteship'];
		$show_gmap = false;
		$message = "";
		if ($biteship_shipping != null) {
			$biteship_options = $biteship_shipping->get_options();
			$shipping_service_enabled = $biteship_options['shipping_service_enabled'];
			if ( !is_array($shipping_service_enabled ) ) {
				$shipping_service_enabled = array();
			}
			$show_gmap = false;
			$has_gojek = false;
			$has_grab = false;
			foreach($shipping_service_enabled as $service) {
				$courier = explode("/", $service)[0];
				if ($courier == "gojek") {
					$has_gojek = true;
					$show_gmap = true;
				} else if ($courier == "grab") {
					$has_grab = true;
					$show_gmap = true;
				}
			}

			$message_var = "";
			if ($has_gojek && $has_grab) {
				$message_var = "Gojek/Grab";
			} else if ($has_gojek) {
				$message_var = "Gojek";
			} else if ($has_grab) {
				$message_var = "Grab";
			}
			$message = "Pick map point to enable shipping using " . $message_var;	
		}

		require_once BITESHIP_PLUGIN_PATH . 'public/views/checkout.php';
	}

	public function custom_position_field( $fields ) {
		$fields['shipping']['position'] = array(
			'type' => 'text',
			'class' => array('input-position'),
		);

		return $fields;
	}

	public function handle_shipping_position( $packages ) {
		$latitude = '';
		$longitude = '';
		if ( isset($_POST['post_data']) ) {
			parse_str($_POST['post_data'], $post_data);
		} else {
			$post_data = $_POST; // fallback for final checkout (non-ajax)
		}

		if ( isset($post_data['position']) ) {
			$position = sanitize_text_field( $post_data['position'] );
			$tmp = explode(',', $position);
			if (count($tmp) > 1) {
				$latitude = $tmp[0];
				$longitude = $tmp[1];
			}
		}

		$packages[0]['destination']['latitude'] = $latitude;
		$packages[0]['destination']['longitude'] = $longitude;
		return $packages;
	}
}
